call plug#begin('~/.vim/plugged')

Plug 'tpope/vim-sensible'
Plug 'tpope/vim-sleuth'
Plug 'tpope/vim-fugitive'
Plug 'tpope/vim-surround'
Plug 'bling/vim-airline'
Plug 'mhinz/vim-signify'
Plug 'mhinz/vim-grepper'
Plug 'sjl/gundo.vim'
Plug 'Shougo/unite.vim'
Plug 'EinfachToll/DidYouMean'

Plug 'vim-scripts/dbext.vim', {'for': 'sql'}
Plug 'cespare/vim-toml'

if executable('lein')
  Plug 'guns/vim-sexp'
  Plug 'tpope/vim-sexp-mappings-for-regular-people'
  Plug 'tpope/vim-fireplace', {'for': 'clojure'}
endif

if executable('sbt')
  Plug 'derekwyatt/vim-scala'
endif

if executable('kotlin')
  Plug 'udalov/kotlin-vim'
endif

if executable('ansible')
  Plug 'pearofducks/ansible-vim'
endif

if executable('rustc')
  Plug 'rust-lang/rust.vim'
endif

if executable('npm')
  Plug 'pangloss/vim-javascript'
  Plug 'mxw/vim-jsx'
endif

Plug 'trusktr/seti.vim'

call plug#end()

" ----- plugin settings ------
let g:signify_vcs_list = [ 'git' ]

let g:jsx_ext_required = 0

call unite#filters#matcher_default#use(['matcher_fuzzy'])

" use ag if present
if executable('ag')
  " we run a wrapper around ag so it respects agignore
  let g:unite_source_rec_async_command = ['sh', expand('~/.vim/ag-unite')]
  let g:grepper = { 'tools': ['ag', 'git', 'grep'] }
endif

autocmd FileType unite call s:unite_settings()
function! s:unite_settings()
  " Enable navigation with control-j and control-k in insert mode
  imap <buffer> <C-j>   <Plug>(unite_select_next_line)
  imap <buffer> <C-k>   <Plug>(unite_select_previous_line)
endfunction

" ------ settings ------

" change terminal title
set title

" xdg-ish settings
set undodir=~/.cache/vim/undo,.
set backupdir=~/.cache/vim/backup,.
set directory=~/.cache/vim/swap,.

" default indentation
set shiftwidth=2
set tabstop=4
set expandtab

" permanent undo history
set undofile

" allow switching buffers
set hidden

" always scroll
set scrolloff=99

" show line numbers
set number
set relativenumber

" set wordwrap indent
set nowrap
set linebreak
if exists("&breakindent")
  set breakindent
  set breakindentopt=shift:2,sbr
endif

" show whitespace
set list

" make a new copy of the file for backup
" setting to no or auto messes with filewatchers
set backupcopy=yes

" ------ key bindings ------

" ; as :
nnoremap ; :
vnoremap ; :

" allow <ESC> via jj in insert mode
inoremap jj <ESC>

" fix j/k with wraps
nnoremap j gj
nnoremap k gk
vnoremap j gj
vnoremap k gk

" opening unite
nnoremap <C-P> :Unite -no-split -buffer-name=files -start-insert file_rec<cr>
nnoremap <C-O> :Unite -no-split -buffer-name=buffers -start-insert buffer<cr>

imap <C-W> <ESC><C-W>
nnoremap <C-T>h :tabprev<cr>
nnoremap <C-T>l :tabnext<cr>

" grepper
nnoremap <Leader>g :Grepper<CR>

" ------ nvim specific ------
if has('nvim')
  let g:airline_theme_patch_func = 'AirlineThemePatch'
  function! AirlineThemePatch(palette)
    let a:palette.terminal = copy(a:palette.insert)
  endfunction

  let g:terminal_scrollback_buffer_size = 10000

  tnoremap <C-j><C-j> <C-\><C-N>
  tmap <C-T> <C-\><C-N><C-T>
  tmap <C-W> <C-\><C-N><C-W>
  tmap <C-P> <C-\><C-N><C-P>
  tmap <C-O> <C-\><C-N><C-O>

  tnoremap <C-V><C-V><C-V> <C-V><C-V>
  tnoremap <C-V><C-V> <C-V>
  tnoremap <C-V> <C-\><C-N>"+pi

  nnoremap <C-P> :Unite -no-split -buffer-name=files -start-insert file_rec/neovim<cr>

  autocmd BufWinEnter,WinEnter term://* startinsert
endif

augroup formatting
  au!
  " strip trailing whitespace
  autocmd BufWritePre * :%s/\s\+$//e
augroup end

colorscheme seti

if filereadable(expand('~/.vimrc.local'))
  source ~/.vimrc.local
endif
